"use strict";
//  Copyright (C) 2018 Zilliqa
//
//  This file is part of Zilliqa-Javascript-Library.
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <https://www.gnu.org/licenses/>.
Object.defineProperty(exports, "__esModule", { value: true });
exports.randomBytes = void 0;
/**
 * randomBytes
 *
 * Uses JS-native CSPRNG to generate a specified number of bytes.
 * NOTE: this method throws if no PRNG is available.
 *
 * @param {number} bytes
 * @returns {string}
 */
exports.randomBytes = function (bytes) {
    var _a;
    var b = Buffer.allocUnsafe(bytes);
    var n = b.byteLength;
    var crypto = global.crypto;
    if (crypto === undefined) {
        // @ts-ignore
        // for IE 11
        crypto = global.msCrypto;
    }
    var isNodeEnv = typeof ((_a = process === null || process === void 0 ? void 0 : process.versions) === null || _a === void 0 ? void 0 : _a.node) === 'string';
    if (isNodeEnv) {
        // For node enviroment, use sodium-native
        // https://paragonie.com/blog/2016/05/how-generate-secure-random-numbers-in-various-programming-languages#nodejs-csprng
        var sodium = require('sodium-native');
        sodium.randombytes_buf(b);
    }
    else if (crypto && crypto.getRandomValues) {
        // For browser or web worker enviroment, use window.crypto.getRandomValues()
        // https://paragonie.com/blog/2016/05/how-generate-secure-random-numbers-in-various-programming-languages#js-csprng
        // limit of getRandomValues()
        // The requested length exceeds 65536 bytes.
        // https://developer.mozilla.org/en-US/docs/Web/API/Crypto/getRandomValues#exceptions
        var MAX_BYTES = 65536;
        for (var i = 0; i < n; i += MAX_BYTES) {
            crypto.getRandomValues(new Uint8Array(b.buffer, i + b.byteOffset, Math.min(n - i, MAX_BYTES)));
        }
    }
    else {
        throw new Error('No secure random number generator available');
    }
    var randBz = new Uint8Array(b.buffer, b.byteOffset, b.byteLength / Uint8Array.BYTES_PER_ELEMENT);
    var randStr = '';
    for (var i = 0; i < bytes; i++) {
        randStr += ('00' + randBz[i].toString(16)).slice(-2);
    }
    return randStr;
};
//# sourceMappingURL=random.js.map